Package: BioQC
Type: Package
Title: Detect tissue heterogeneity in expression profiles with gene
        sets
Version: 1.40.0
Date: 2021-08-18
Authors@R: c(
    person("Jitao David", "Zhang", email="jitao_david.zhang@roche.com", role = c("cre", "aut")), 
    person("Laura", "Badi", role="aut"),
    person("Gregor", "Sturm", email="mail@gregor-sturm.de", role=c("aut")),
    person("Roland", "Ambs", role="aut"),
    person("Iakov", "Davydov", role="aut"))
Description: BioQC performs quality control of high-throughput expression data
    based on tissue gene signatures. It can detect tissue heterogeneity in gene
    expression data. The core algorithm is a Wilcoxon-Mann-Whitney test that is
    optimised for high performance.
biocViews: GeneExpression,QualityControl,StatisticalMethod,
        GeneSetEnrichment
Depends: R (>= 3.5.0), Biobase
Imports: edgeR, Rcpp, methods, stats, utils
Collate: AllClasses.R AllMethods.R as.GmtList.R GmtList-funcs.R
        readSignedGmt.R uniqGenesetsByNamespace.R appendGmtList.R
        utils.R entropy.R gini.R readGmt.R matchGenes.R wmwTest.R
        readCurrentSignatures.R prettySigNames.R leadingEdge.R
Suggests: testthat, knitr, rmarkdown, lattice, latticeExtra,
        rbenchmark, gplots, gridExtra, org.Hs.eg.db, hgu133plus2.db,
        ggplot2, reshape2, plyr, ineq, covr, limma, RColorBrewer
LinkingTo: Rcpp
VignetteBuilder: knitr
License: GPL (>=3) + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.1.1
URL: https://accio.github.io/BioQC
BugReports: https://accio.github.io/BioQC/issues
git_url: https://git.bioconductor.org/packages/BioQC
git_branch: RELEASE_3_23
git_last_commit: b7d663f
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: yes
Packaged: 2026-04-28 20:56:42 UTC; biocbuild
Author: Jitao David Zhang [cre, aut],
  Laura Badi [aut],
  Gregor Sturm [aut],
  Roland Ambs [aut],
  Iakov Davydov [aut]
Maintainer: Jitao David Zhang <jitao_david.zhang@roche.com>
