# Generated by roxygen2: do not edit by hand

S3method("[",GmtList)
S3method("[[",GmtList)
export("gsNamespace<-")
export(GmtList)
export(SignedGenesets)
export(SignedIndexList)
export(absLog10p)
export(appendGmtList)
export(as.GmtList)
export(entropy)
export(entropyDiversity)
export(entropySpecificity)
export(filterBySize)
export(filterPmat)
export(getLeadingEdgeIndexFromMatrix)
export(getLeadingEdgeIndexFromVector)
export(gini)
export(gmtlist2signedGenesets)
export(gsDesc)
export(gsGeneCount)
export(gsGenes)
export(gsName)
export(gsNamespace)
export(gsSize)
export(hasNamespace)
export(isValidBaseIndexList)
export(isValidGmtList)
export(isValidIndexList)
export(isValidSignedGenesets)
export(isValidSignedIndexList)
export(prettySigNames)
export(readCurrentSignatures)
export(readGmt)
export(readSignedGmt)
export(sampleSpecialization)
export(setDescAsNamespace)
export(setNamespace)
export(simplifyMatrix)
export(uniqGenesetsByNamespace)
export(wmwLeadingEdge)
export(wmwTestInR)
exportClasses(GmtList)
exportClasses(IndexList)
exportClasses(SignedGenesets)
exportClasses(SignedIndexList)
exportMethods("offset<-")
exportMethods(IndexList)
exportMethods(SignedIndexList)
exportMethods(matchGenes)
exportMethods(offset)
exportMethods(show)
exportMethods(wmwTest)
import(Rcpp)
importClassesFrom(Biobase,eSet)
importClassesFrom(edgeR,DGEList)
importFrom(Biobase,exprs)
importFrom(Biobase,fData)
importFrom(Biobase,featureNames)
importFrom(Rcpp,sourceCpp)
importFrom(methods,is)
importFrom(methods,new)
importFrom(stats,median)
importFrom(stats,pt)
importFrom(stats,wilcox.test)
importFrom(utils,head)
useDynLib(BioQC, .registration=TRUE, .fixes="C_")
