% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllClasses.R
\docType{class}
\name{GmtList-class}
\alias{GmtList-class}
\title{An S4 class to hold geneset in the GMT file in a list, each item in the list 
is in in turn a list containing following items: name, desc, and genes.}
\description{
An S4 class to hold geneset in the GMT file in a list, each item in the list 
is in in turn a list containing following items: name, desc, and genes.
}
