% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wmwTest.R
\name{valTypes}
\alias{valTypes}
\title{prints the options of valTypes of wmwTest}
\usage{
valTypes()
}
\description{
prints the options of valTypes of wmwTest
}
